diff --git a/libavformat/demux.c b/libavformat/demux.c
index c1c9422ac0..b314477e47 100644
--- a/libavformat/demux.c
+++ b/libavformat/demux.c
@@ -217,6 +217,56 @@ FF_ENABLE_DEPRECATION_WARNINGS
     return 0;
 }
 
+
+int avformat_alloc_input_context(AVFormatContext** avctx, const AVInputFormat* iformat,
+    const char* format)
+{
+    AVFormatContext* s = avformat_alloc_context();
+    int ret = 0;
+
+    *avctx = NULL;
+    if (!s)
+        goto nomem;
+
+    if (!iformat) {
+        if (format) {
+            iformat = av_find_input_format(format);
+            if (!iformat) {
+                av_log(s, AV_LOG_ERROR, "Requested input format '%s' not found\n", format);
+                ret = AVERROR(EINVAL);
+                goto error;
+            }
+        }
+        else {
+            av_log(s, AV_LOG_ERROR, "You should provide an input format or the name of an input format when calling this function\n");
+            ret = AVERROR(EINVAL);
+            goto error;
+        }
+    }
+
+    s->iformat = iformat;
+    if (s->iformat->priv_data_size > 0) {
+        s->priv_data = av_mallocz(s->iformat->priv_data_size);
+        if (!s->priv_data)
+            goto nomem;
+        if (s->iformat->priv_class) {
+            *(const AVClass**)s->priv_data = s->iformat->priv_class;
+            av_opt_set_defaults(s->priv_data);
+        }
+    }
+    else
+        s->priv_data = NULL;
+
+    *avctx = s;
+    return 0;
+nomem:
+    av_log(s, AV_LOG_ERROR, "Out of memory\n");
+    ret = AVERROR(ENOMEM);
+error:
+    avformat_free_context(s);
+    return ret;
+}
+
 int avformat_open_input(AVFormatContext **ps, const char *filename,
                         const AVInputFormat *fmt, AVDictionary **options)
 {
@@ -233,8 +283,14 @@ int avformat_open_input(AVFormatContext **ps, const char *filename,
         av_log(NULL, AV_LOG_ERROR, "Input context has not been properly allocated by avformat_alloc_context() and is not NULL either\n");
         return AVERROR(EINVAL);
     }
-    if (fmt)
+    if (fmt) {
+        if (s->iformat) {
+            if (s->iformat->priv_class && s->priv_data)
+                av_opt_free(s->priv_data);
+            av_freep(&s->priv_data);
+        }
         s->iformat = fmt;
+    }
 
     if (options)
         av_dict_copy(&tmp, *options, 0);
@@ -245,7 +301,9 @@ int avformat_open_input(AVFormatContext **ps, const char *filename,
     if ((ret = av_opt_set_dict(s, &tmp)) < 0)
         goto fail;
 
-    if (!(s->url = av_strdup(filename ? filename : ""))) {
+    if (filename && s->url)
+        av_freep(&s->url);
+    if (!s->url && !(s->url = av_strdup(filename ? filename : ""))) {
         ret = AVERROR(ENOMEM);
         goto fail;
     }
@@ -288,12 +346,16 @@ int avformat_open_input(AVFormatContext **ps, const char *filename,
 
     s->duration = s->start_time = AV_NOPTS_VALUE;
 
-    /* Allocate private data. */
+    /* Allocate private data and set options. */
     if (s->iformat->priv_data_size > 0) {
-        if (!(s->priv_data = av_mallocz(s->iformat->priv_data_size))) {
-            ret = AVERROR(ENOMEM);
-            goto fail;
+        /* allocate if not already allocated */
+        if (!s->priv_data) {
+            if (!(s->priv_data = av_mallocz(s->iformat->priv_data_size))) {
+                ret = AVERROR(ENOMEM);
+                goto fail;
+            }
         }
+        /* Overwrite any options already set (1: back to defaults, 2: apply options, if any) */
         if (s->iformat->priv_class) {
             *(const AVClass **) s->priv_data = s->iformat->priv_class;
             av_opt_set_defaults(s->priv_data);
diff --git a/libavformat/version.h b/libavformat/version.h
index 1ab39022c2..7cd09f7ef8 100644
--- a/libavformat/version.h
+++ b/libavformat/version.h
@@ -32,7 +32,7 @@
 
 #include "version_major.h"
 
-#define LIBAVFORMAT_VERSION_MINOR  23
+#define LIBAVFORMAT_VERSION_MINOR  24
 #define LIBAVFORMAT_VERSION_MICRO 100
 
 #define LIBAVFORMAT_VERSION_INT AV_VERSION_INT(LIBAVFORMAT_VERSION_MAJOR, \

diff --git a/libavformat/avformat.h b/libavformat/avformat.h
index 027a914e13..004d81640f 100644
--- a/libavformat/avformat.h
+++ b/libavformat/avformat.h
@@ -2024,6 +2024,26 @@ AVProgram *av_new_program(AVFormatContext *s, int id);
  * @}
  */
 
+ /**
+  * Allocate an AVFormatContext for an input format.
+  * avformat_free_context() can be used to free the context and
+  * everything allocated by the framework within it. NB: in general
+  * the correct format cannot be known (unless the user has extra
+  * information) until the file is opened. If forcing a format by
+  * this method, but it turns out not to match the file's format
+  * upon avformat_open_input(), the latter will throw an error.
+  *
+  * @param *ctx is set to the created format context, or to NULL in
+  * case of failure
+  * @param iformat format to use for allocating the context, if NULL
+  * format_name is used instead
+  * @param format_name the name of input format to use for allocating the
+  * context
+  * @return >= 0 in case of success, a negative AVERROR code in case of
+  * failure
+  */
+int avformat_alloc_input_context(AVFormatContext **ctx, const AVInputFormat *iformat,
+                                 const char *format_name);
 
 /**
  * Allocate an AVFormatContext for an output format.
@@ -2119,9 +2139,9 @@ int av_probe_input_buffer(AVIOContext *pb, const AVInputFormat **fmt,
  * Open an input stream and read the header. The codecs are not opened.
  * The stream must be closed with avformat_close_input().
  *
  * @param ps Pointer to user-supplied AVFormatContext (allocated by avformat_alloc_context).
  *           May be a pointer to NULL, in which case an AVFormatContext is allocated by this
  *           function and written into ps.
  *           Note that a user-supplied AVFormatContext will be freed on failure.
  * @param url URL of the stream to open.
  * @param fmt If non-NULL, this parameter forces a specific input format.
@@ -2129,6 +2149,9 @@ int av_probe_input_buffer(AVIOContext *pb, const AVInputFormat **fmt,
  * @param options  A dictionary filled with AVFormatContext and demuxer-private options.
  *                 On return this parameter will be destroyed and replaced with a dict containing
  *                 options that were not found. May be NULL.
+ *                 Note that if a AVFormatContext allocated by avformat_alloc_input_context
+ *                 is provided, any demuxer-private options will be overwritten by their defaults
+ *                 before applying this new set of demuxer-private options, if any. 
  *
  * @return 0 on success, a negative AVERROR on failure.
  *
@@ -2119,9 +2139,9 @@ int av_probe_input_buffer(AVIOContext *pb, const AVInputFormat **fmt,
  * The stream must be closed with avformat_close_input().
  *
- * @param ps Pointer to user-supplied AVFormatContext (allocated by avformat_alloc_context).
- *           May be a pointer to NULL, in which case an AVFormatContext is allocated by this
- *           function and written into ps.
+ * @param ps Pointer to user-supplied AVFormatContext (allocated by avformat_alloc_context, or
+ *           avformat_alloc_input_context). May be a pointer to NULL, in which case an
+ *           AVFormatContext is allocated by this function and written into ps.
  *           Note that a user-supplied AVFormatContext will be freed on failure.
  * @param url URL of the stream to open.